@@ -28,9 +28,12 @@
 #include "ContextMenu.h"
 #include "Piano.h"
 #include "Tools.h"
+#include "Tracker.h"
+#include "SampleEditor.h"
 #include "TrackerConfig.h"
 #include "PlayerController.h"
 #include "DialogBase.h"
+#include "FilterParameters.h"
 
 #include <algorithm>
 #include <math.h>
@@ -64,6 +67,7 @@ SampleEditorControl::SampleEditorControl(pp_int32 id,
 										 EventListenerInterface* eventListener, 
 										 const PPPoint& location, 
 										 const PPSize& size, 
+										 Tracker& tracker,
 										 bool border/*= true*/) :
 	PPControl(id, parentScreen, eventListener, location, size),
 	border(border),
@@ -117,9 +121,14 @@ SampleEditorControl::SampleEditorControl(pp_int32 id,
 	subMenuAdvanced = new PPContextMenu(5, parentScreen, this, PPPoint(0,0), TrackerConfig::colorThemeMain);
 	subMenuAdvanced->setSubMenu(true);
 	subMenuAdvanced->addEntry("Volume boost" PPSTR_PERIODS, MenuCommandIDVolumeBoost);
-	subMenuAdvanced->addEntry("Volume fade" PPSTR_PERIODS, MenuCommandIDVolumeFade);
 	subMenuAdvanced->addEntry("Normalize", MenuCommandIDNormalize);
+	subMenuAdvanced->addEntry("Compress", MenuCommandIDCompress);
 	subMenuAdvanced->addEntry(seperatorStringLarge, -1);
+	subMenuAdvanced->addEntry("Fade in" PPSTR_PERIODS, MenuCommandIDVolumeFadeIn);
+	subMenuAdvanced->addEntry("Fade out" PPSTR_PERIODS, MenuCommandIDVolumeFadeOut);
+	subMenuAdvanced->addEntry("Fade custom" PPSTR_PERIODS, MenuCommandIDVolumeFade);
+	subMenuAdvanced->addEntry(seperatorStringLarge, -1);
+	subMenuAdvanced->addEntry("Loop fold" PPSTR_PERIODS, MenuCommandIDVolumeFold);
 	subMenuAdvanced->addEntry("Backwards", MenuCommandIDReverse);
 	subMenuAdvanced->addEntry("Cross-fade", MenuCommandIDXFade);
 	subMenuAdvanced->addEntry(seperatorStringLarge, -1);
@@ -144,6 +153,8 @@ SampleEditorControl::SampleEditorControl(pp_int32 id,
 	subMenuXPaste->addEntry("Phase Modulate", MenuCommandIDPHPaste);
 	subMenuXPaste->addEntry("Flanger", MenuCommandIDFLPaste);
 	subMenuXPaste->addEntry("Selective EQ" PPSTR_PERIODS, MenuCommandIDSelectiveEQ10Band);
+	subMenuXPaste->addEntry("Capture pattern" PPSTR_PERIODS, MenuCommandIDCapturePattern);
+
 
 	subMenuPT = new PPContextMenu(6, parentScreen, this, PPPoint(0,0), TrackerConfig::colorThemeMain);
 	subMenuPT->addEntry("Boost", MenuCommandIDPTBoost);
@@ -154,6 +165,9 @@ SampleEditorControl::SampleEditorControl(pp_int32 id,
 	subMenuGenerators->addEntry("Square" PPSTR_PERIODS, MenuCommandIDGenerateSquare);
 	subMenuGenerators->addEntry("Triangle" PPSTR_PERIODS, MenuCommandIDGenerateTriangle);
 	subMenuGenerators->addEntry("Sawtooth" PPSTR_PERIODS, MenuCommandIDGenerateSawtooth);
+	subMenuGenerators->addEntry("Half Sine" PPSTR_PERIODS, MenuCommandIDGenerateHalfSine);
+	subMenuGenerators->addEntry("Absolute Sine" PPSTR_PERIODS, MenuCommandIDGenerateAbsoluteSine);
+	subMenuGenerators->addEntry("Quarter Sine" PPSTR_PERIODS, MenuCommandIDGenerateQuarterSine);
 	subMenuGenerators->addEntry("Silence" PPSTR_PERIODS, MenuCommandIDGenerateSilence);
 	
 	// build context menu
@@ -168,6 +182,7 @@ SampleEditorControl::SampleEditorControl(pp_int32 id,
 	editMenuControl->addEntry("Paste", MenuCommandIDPaste);
 	editMenuControl->addEntry("Crop", MenuCommandIDCrop);
 	editMenuControl->addEntry("Range all", MenuCommandIDSelectAll);
+	editMenuControl->addEntry("Loop range", MenuCommandIDLoopRange);
 	editMenuControl->addEntry(seperatorStringMed, -1);
 	editMenuControl->addEntry("Advanced   \x10", 0xFFFF, subMenuAdvanced);
 	editMenuControl->addEntry("Ext. Paste \x10", 0xFFFF, subMenuXPaste);
@@ -177,6 +192,7 @@ SampleEditorControl::SampleEditorControl(pp_int32 id,
 	// Create tool handler responder
 	toolHandlerResponder = new ToolHandlerResponder(*this);
 	dialog = NULL;	
+	this->tracker = (Tracker *)&tracker;
 	
 	resetLastValues();
 }
@@ -1437,6 +1453,14 @@ void SampleEditorControl::rangeAll(bool updateNotify/* = false*/)
 		notifyUpdate();
 }
 
+void SampleEditorControl::loopRange(bool updateNotify/* = false*/)
+{	
+	sampleEditor->loopRange();
+
+	if (updateNotify)
+		notifyUpdate();
+}
+
 void SampleEditorControl::rangeClear(bool updateNotify/* = false*/)
 {
 	sampleEditor->resetSelection();
@@ -1664,10 +1688,15 @@ void SampleEditorControl::invokeContextMenu(const PPPoint& p, bool translatePoin
 	editMenuControl->setState(MenuCommandIDCut, !hasValidSelection());
 	editMenuControl->setState(MenuCommandIDCrop, !hasValidSelection());
 	editMenuControl->setState(MenuCommandIDSelectAll, isEmptySample);
+	editMenuControl->setState(MenuCommandIDLoopRange, !hasValidSelection());
 	
 	// update submenu states
 	subMenuAdvanced->setState(MenuCommandIDNormalize, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDCompress, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDVolumeFade, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDVolumeFadeIn, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDVolumeFadeOut, isEmptySample);
+	subMenuAdvanced->setState(MenuCommandIDVolumeFold, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDVolumeBoost, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDReverse, isEmptySample);
 	subMenuAdvanced->setState(MenuCommandIDXFade, !sampleEditor->isValidxFadeSelection() || isEmptySample || !sampleEditor->getLoopType());
@@ -1695,6 +1724,9 @@ void SampleEditorControl::invokeContextMenu(const PPPoint& p, bool translatePoin
 	subMenuGenerators->setState(MenuCommandIDGenerateSquare, isEmptySample);
 	subMenuGenerators->setState(MenuCommandIDGenerateTriangle, isEmptySample);
 	subMenuGenerators->setState(MenuCommandIDGenerateSawtooth, isEmptySample);
+	subMenuGenerators->setState(MenuCommandIDGenerateHalfSine, isEmptySample);
+	subMenuGenerators->setState(MenuCommandIDGenerateAbsoluteSine, isEmptySample);
+	subMenuGenerators->setState(MenuCommandIDGenerateQuarterSine, isEmptySample);
 	subMenuGenerators->setState(MenuCommandIDGenerateSilence, isEmptySample);
 
 	parentScreen->setContextMenuControl(editMenuControl);
@@ -1708,6 +1740,7 @@ void SampleEditorControl::hideContextMenu()
 
 void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 {
+
 	switch (commandId)
 	{
 		// cut
@@ -1770,11 +1803,20 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			rangeAll(true);
 			break;
 					
+		// set loop to current selection
+		case MenuCommandIDLoopRange:
+			loopRange(true);
+			break;	
+
 		// Invoke tools
 		case MenuCommandIDNew:
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeNew);
 			break;
 
+		case MenuCommandIDCapturePattern:
+			tracker->eventKeyDownBinding_InvokePatternCapture();
+			break;
+
 		case MenuCommandIDVolumeBoost:
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeVolume);
 			break;
@@ -1783,6 +1825,27 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeFade);
 			break;
 
+		case MenuCommandIDVolumeFadeOut:{
+			FilterParameters par(2);
+			par.setParameter(0, FilterParameters::Parameter(1.0f));
+			par.setParameter(1, FilterParameters::Parameter(0.0f));
+			sampleEditor->tool_scaleSample(&par);
+			break;
+		}
+
+		case MenuCommandIDVolumeFadeIn:{
+			FilterParameters par(2);
+			par.setParameter(0, FilterParameters::Parameter(0.0f));
+			par.setParameter(1, FilterParameters::Parameter(1.0f));
+			sampleEditor->tool_scaleSample(&par);
+			break;
+		}
+
+		case MenuCommandIDVolumeFold:{
+			sampleEditor->tool_foldSample(NULL);
+			break;
+		}
+
 		case MenuCommandIDResample:
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeResample);
 			break;
@@ -1795,6 +1858,10 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			sampleEditor->tool_normalizeSample(NULL);
 			break;
 
+		case MenuCommandIDCompress:
+			sampleEditor->tool_compressSample(NULL);
+			break;
+
 		case MenuCommandIDReverse:
 			sampleEditor->tool_reverseSample(NULL);
 			break;
@@ -1863,6 +1930,18 @@ void SampleEditorControl::executeMenuCommand(pp_int32 commandId)
 			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeGenerateSawtooth);
 			break;
 
+		case MenuCommandIDGenerateHalfSine:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeGenerateHalfSine);
+			break;
+
+		case MenuCommandIDGenerateAbsoluteSine:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeGenerateAbsoluteSine);
+			break;
+
+		case MenuCommandIDGenerateQuarterSine:
+			invokeToolParameterDialog(ToolHandlerResponder::SampleToolTypeGenerateQuarterSine);
+			break;
+
 	}
 }
 
